/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*

    Copyright (C) 2012-2015  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Gettext = imports.gettext;
const Lang = imports.lang;

const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;
const Gvc = imports.gi.Gvc;
const Shell = imports.gi.Shell;
const St = imports.gi.St;
const Tp = imports.gi.TelepathyGLib;

const ExtensionUtils = imports.misc.extensionUtils;

const Main = imports.ui.main;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;

const Extension = ExtensionUtils.getCurrentExtension();

const Convenience = Extension.imports.convenience;
const PanelTools = Extension.imports.panelTools;
const SystemTools = Extension.imports.systemTools;

const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

/*
    PanelManager - AggregateMenu
 */
const StatusArea = Main.panel.statusArea || Main.panel._statusArea;

const KEY_SHOW_STATUSAREA = 'show-statusarea';
const KEY_SHOW_STATUSMENU = 'show-statusmenu';

const STATUSAREA_DEFAULT = 0;
const STATUSAREA_ADVANCED = 1;
const STATUSAREA_ALTERNATE = 2;

const STATUSMENU_DEFAULT = 0;
const STATUSMENU_ICON = 1;

let _prototypeUpdateMultiUser;

function _overwriteSystem(enable) {
    const PanelManager = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);

    if (enable) {
        _prototypeUpdateMultiUser = imports.ui.status.system.Indicator.prototype._updateMultiUser;
        imports.ui.status.system.Indicator.prototype._updateMultiUser = function () {
            _prototypeUpdateMultiUser.call(this);
            let visible = this._switchUserSubMenu.actor.visible;
            if (!visible && PanelManager.get_enum(KEY_SHOW_STATUSAREA) !== STATUSAREA_DEFAULT) this._switchUserSubMenu.actor.visible = this._lockScreenAction.visible;
        };
    } else {
        imports.ui.status.system.Indicator.prototype._updateMultiUser = _prototypeUpdateMultiUser;
    }
}

const SystemIndicator = new Lang.Class({
    Name: 'SystemIndicator',

    _init: function() {
        _overwriteSystem(true);
        this.indicators = new St.Icon({ icon_name: 'system-shutdown-symbolic', style_class: 'system-status-icon' });
        this._itemLockScreen = new PanelTools.PopupMenuItem(_("Lock Screen"));
        this._itemLockScreen.connect('activate', Lang.bind(StatusArea.aggregateMenu._system, SystemTools.activateLockScreen));
        StatusArea.aggregateMenu._system._switchUserSubMenu.menu.addMenuItem(this._itemLockScreen, 1);
        this._actionPanelManagerSettings = StatusArea.aggregateMenu._system._createActionButton('preferences-other-symbolic', 'Settings for PanelManager');
        this._actionPanelManagerSettings.connect('clicked', Lang.bind(StatusArea.aggregateMenu._system, SystemTools.activatePanelManagerSettings));
        this._actionGNOMEShellRestart = StatusArea.aggregateMenu._system._createActionButton('view-refresh-symbolic', _("GNOME Shell restart"));
        this._actionGNOMEShellRestart.connect('clicked', Lang.bind(StatusArea.aggregateMenu._system, SystemTools.activateGNOMEShellRestart));
        this._actionTerminal = StatusArea.aggregateMenu._system._createActionButton('utilities-terminal-symbolic', _("Terminal"));
        this._actionTerminal.connect('clicked', Lang.bind(StatusArea.aggregateMenu._system, SystemTools.activateTerminal));
        this._actionSuspend = StatusArea.aggregateMenu._system._createActionButton('media-playback-pause-symbolic', _("Suspend"));
        this._actionSuspend.connect('clicked', Lang.bind(StatusArea.aggregateMenu._system, StatusArea.aggregateMenu._system._onSuspendClicked));
        this._actionHibernate = StatusArea.aggregateMenu._system._createActionButton('media-playback-stop-symbolic', _("Hibernate"));
        this._actionHibernate.connect('clicked', Lang.bind(StatusArea.aggregateMenu._system, SystemTools.activateHibernate));
        this._actionPowerOff = StatusArea.aggregateMenu._system._createActionButton('system-shutdown-symbolic', _("Power Off"));
        this._actionPowerOff.connect('clicked', Lang.bind(StatusArea.aggregateMenu._system, SystemTools.activatePowerOff));
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = this._settings.connect('changed::'+KEY_SHOW_STATUSAREA, Lang.bind(this, this._updateVisibility));
        this._updateVisibility();
        this._connectSystemMenu = StatusArea.aggregateMenu._system.menu.connect('open-state-changed', Lang.bind(this, function(menu, open) {
            if (!open) return;
            this._updateVisibility();
        }));
    },

    _updateActions: function(showStatusArea) {
        let actions = StatusArea.aggregateMenu._system._actionsItem.actor.get_children();
        for (let i = 1; i < actions.length; i++) {
            StatusArea.aggregateMenu._system._actionsItem.actor.remove_actor(actions[i]);
        }
        if (showStatusArea === STATUSAREA_DEFAULT) StatusArea.aggregateMenu._system._actionsItem.actor.add(StatusArea.aggregateMenu._system._settingsAction, { expand: true, x_fill: false });
        if (showStatusArea !== STATUSAREA_ADVANCED) StatusArea.aggregateMenu._system._actionsItem.actor.add(this._actionPanelManagerSettings, { expand: true, x_fill: false });
        StatusArea.aggregateMenu._system._actionsItem.actor.add(StatusArea.aggregateMenu._system._orientationLockAction, { expand: true, x_fill: false });
        if (showStatusArea !== STATUSAREA_DEFAULT) StatusArea.aggregateMenu._system._actionsItem.actor.add(this._actionGNOMEShellRestart, { expand: true, x_fill: false });
        if (showStatusArea !== STATUSAREA_DEFAULT) StatusArea.aggregateMenu._system._actionsItem.actor.add(this._actionTerminal, { expand: true, x_fill: false });
        if (showStatusArea !== STATUSAREA_DEFAULT) StatusArea.aggregateMenu._system._actionsItem.actor.add(this._actionSuspend, { expand: true, x_fill: false });
        if (showStatusArea !== STATUSAREA_DEFAULT) StatusArea.aggregateMenu._system._actionsItem.actor.add(this._actionHibernate, { expand: true, x_fill: false });
        if (showStatusArea !== STATUSAREA_DEFAULT) StatusArea.aggregateMenu._system._actionsItem.actor.add(this._actionPowerOff, { expand: true, x_fill: false });
        if (showStatusArea === STATUSAREA_DEFAULT) StatusArea.aggregateMenu._system._actionsItem.actor.add(StatusArea.aggregateMenu._system._lockScreenAction, { expand: true, x_fill: false });
        if (showStatusArea === STATUSAREA_DEFAULT) StatusArea.aggregateMenu._system._actionsItem.actor.add(StatusArea.aggregateMenu._system._altSwitcher.actor, { expand: true, x_fill: false });
        StatusArea.aggregateMenu._system._updateOrientationLock();
        StatusArea.aggregateMenu._system._sessionUpdated();
    },

    _updateVisibility: function() {
        let showStatusArea = this._settings.get_enum(KEY_SHOW_STATUSAREA);
        this._updateActions(showStatusArea);
        SystemTools.updateLockScreen.call(StatusArea.aggregateMenu._system, this._itemLockScreen.actor);
        this._actionTerminal.visible = PanelTools.checkApplication('gnome-terminal.desktop');
        SystemTools.updateSuspend.call(StatusArea.aggregateMenu._system, this._actionSuspend);
        SystemTools.updateHibernate.call(StatusArea.aggregateMenu._system, this._actionHibernate);
        SystemTools.updatePowerOff.call(StatusArea.aggregateMenu._system, this._actionPowerOff);
    },

    destroy: function() {
        StatusArea.aggregateMenu._system.menu.disconnect(this._connectSystemMenu);
        this._settings.disconnect(this._connectSettings);
        _overwriteSystem(false);
        this._updateActions(STATUSAREA_DEFAULT);
        this._actionPowerOff.destroy();
        this._actionHibernate.destroy();
        this._actionSuspend.destroy();
        this._actionTerminal.destroy();
        this._actionGNOMEShellRestart.destroy();
        this._actionPanelManagerSettings.destroy();
        this._itemLockScreen.destroy();
        this.indicators.destroy();
    }
});

const AggregateMenu = new Lang.Class({
    Name: 'AggregateMenu',

    _init: function() {
        this._separator = new PopupMenu.PopupSeparatorMenuItem();
        this._communicationApplications = new Extension.imports.communications.CommunicationApplicationsSystemIndicator();
        this._basicSettings = new Extension.imports.basicSettings.BasicSettingsSystemIndicator();
        this._system = new SystemIndicator();
        this._createIndicators();
        this._createSubMenu();
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = [
            this._settings.connect('changed::'+KEY_SHOW_STATUSAREA, Lang.bind(this, this._onShowStatusArea)),
            this._settings.connect('changed::'+KEY_SHOW_STATUSMENU, Lang.bind(this, this._onShowStatusMenu))
        ];
        this._onShowStatusArea();
        this._connectMenu = StatusArea.aggregateMenu.menu.connect('open-state-changed', Lang.bind(this, function(menu, open) {
            if (!open) return;
        }));
    },

    _createIndicators: function() {
        StatusArea.aggregateMenu._indicators.insert_child_at_index(this._basicSettings.indicators, StatusArea.aggregateMenu._indicators.get_children().length - 2);
        StatusArea.aggregateMenu.actor.remove_actor(StatusArea.aggregateMenu._indicators);
        this._indicators = new St.BoxLayout();
        StatusArea.aggregateMenu.actor.add_child(this._indicators);
        this._indicators.add_child(StatusArea.aggregateMenu._indicators);
        this._indicators.add_child(this._system.indicators);
    },

    _createSubMenu: function() {
        StatusArea.aggregateMenu.menu.actor.remove_style_class_name('aggregate-menu');
        let menuItems = StatusArea.aggregateMenu.menu._getMenuItems();
        for (let i = menuItems.length - 1; i >= 0; i--) {
            if (menuItems[i] === StatusArea.aggregateMenu._power.menu) {
                StatusArea.aggregateMenu.menu.addMenuItem(this._basicSettings.menu, i + 1);
                StatusArea.aggregateMenu.menu.addMenuItem(this._communicationApplications.menu, i + 1);
                StatusArea.aggregateMenu.menu.addMenuItem(this._separator, i + 1);
            }
        }
    },

    _onShowStatusArea: function() {
        let showStatusArea = this._settings.get_enum(KEY_SHOW_STATUSAREA);
        this._communicationApplications.menu.actor.visible = showStatusArea === STATUSAREA_ALTERNATE;
        this._basicSettings.menu.actor.visible = showStatusArea === STATUSAREA_ALTERNATE;
        this._onShowStatusMenu();
    },

    _onShowStatusMenu: function() {
        let showStatusArea = this._settings.get_enum(KEY_SHOW_STATUSAREA);
        let showStatusMenu = this._settings.get_enum(KEY_SHOW_STATUSMENU);
        StatusArea.aggregateMenu._indicators.visible = showStatusMenu === STATUSMENU_DEFAULT;
        this._basicSettings.indicators.visible = showStatusArea === STATUSAREA_ALTERNATE && showStatusMenu === STATUSMENU_DEFAULT;
        this._system.indicators.visible = showStatusMenu === STATUSMENU_ICON;
    },

    _restoreIndicators: function() {
        this._indicators.remove_actor(StatusArea.aggregateMenu._indicators);;
        StatusArea.aggregateMenu.actor.remove_actor(this._indicators);
        StatusArea.aggregateMenu.actor.add_child(StatusArea.aggregateMenu._indicators);
    },

    _restoreSubMenu: function() {
        StatusArea.aggregateMenu.menu.actor.add_style_class_name('aggregate-menu');
    },

    destroy: function() {
        StatusArea.aggregateMenu.menu.disconnect(this._connectMenu);
        for each(let connection in this._connectSettings) {
            this._settings.disconnect(connection);
        }
        this._system.destroy();
        this._basicSettings.destroy();
        this._communicationApplications.destroy();
        this._separator.destroy();
        this._restoreSubMenu();
        this._restoreIndicators();
    }
});

function getExtension() {
    return StatusArea.aggregateMenu ? new AggregateMenu() : null;
}
