/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*

    Copyright (C) 2012-2015  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Gettext = imports.gettext;
const Lang = imports.lang;

const Shell = imports.gi.Shell;

const ExtensionUtils = imports.misc.extensionUtils;

const Extension = ExtensionUtils.getCurrentExtension();

const Convenience = Extension.imports.convenience;
const PanelTools = Extension.imports.panelTools;

const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

/*
    PanelManager - Favorites
 */
const KEY_SHOW_FAVORITES = 'show-favorites';

const FavoritesIndicator = new Lang.Class({
    Name: 'FavoritesIndicator',
    Extends: PanelTools.Button,

    _init: function() {
        this.parent(1.0, _("Favorites"), { icon_name: 'emblem-favorite-symbolic', text: _("Favorites") });
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = this._settings.connect('changed::'+KEY_SHOW_FAVORITES, Lang.bind(this, this._onShowFavorites));
        this._onShowFavorites();
        this._connectFavorites = global.settings.connect('changed::favorite-apps', Lang.bind(this, this._onFavoriteApps));
        this._onFavoriteApps();
        this.actor.connect('destroy', Lang.bind(this, function() {
            global.settings.disconnect(this._connectFavorites);
            this._settings.disconnect(this._connectSettings);
        }));
    },

    _onFavoriteApps: function() {
        this.menu.removeAll();
        let appList = [];
        let favorites = global.settings.get_strv('favorite-apps');
        for (let i = 0; i < favorites.length; i++) {
            let app = Shell.AppSystem.get_default().lookup_app(favorites[i]);
            if (app) appList.push(app);
        }
        appList.sort(function(a, b) {
            return a.compare_by_name(b);
        });
        for (let i = 0; i < appList.length; i++) {
            this.menu.addMenuItem(new PanelTools.PopupApplicationMenuItem(appList[i], true));
        }
    },

    _onShowFavorites: function() {
        this.updateButton(this._settings.get_enum(KEY_SHOW_FAVORITES));
    }
});

function getExtension() {
    return PanelTools.addToStatusArea('dschaerfFavorites', new FavoritesIndicator(), 1, 'left');
}
