/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*

    Copyright (C) 2012-2015  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Lang = imports.lang;

const Clutter = imports.gi.Clutter;
const St = imports.gi.St;

const ExtensionUtils = imports.misc.extensionUtils;

const Main = imports.ui.main;
const Tweener = imports.ui.tweener;

const Extension = ExtensionUtils.getCurrentExtension();

const Convenience = Extension.imports.convenience;

/*
    PanelManager - HotCorners
 */
const KEY_HOTCORNERS_SENSITIVITY = 'hotcorners-sensitivity';
const KEY_SHOW_HOTCORNER_LEFT = 'show-hotcorner-left';
const KEY_SHOW_HOTCORNER_RIGHT = 'show-hotcorner-right';

const HotCornerSet = new Lang.Class({
    Name: 'HotCornerSet',

    _init: function(cornerSize, monitorWidth) {
        this._entered = false;
        this.actor = new Clutter.Group({ name: 'hot-corners-environs', width: cornerSize + 2, height: cornerSize + 2, reactive: true });
        this._leftCorner = new Clutter.Rectangle({ name: 'hot-corner-left', width: cornerSize, height: cornerSize, opacity: 0, reactive: true });
        this._leftCorner._delegate = this;
        this.actor.add_actor(this._leftCorner);
        this._leftCorner.set_position(0, 0);
        this._rightCorner = new Clutter.Rectangle({ name: 'hot-corner-right', width: cornerSize, height: cornerSize, opacity: 0, reactive: true });
        this._rightCorner._delegate = this;
        this.actor.add_actor(this._rightCorner);
        this._rightCorner.set_position(monitorWidth - cornerSize, 0);
        this._activationTime = 0;
        this.actor.connect('button-release-event', Lang.bind(this, this._onButtonReleaseEvent));
        this.actor.connect('leave-event', Lang.bind(this, this._onLeaveEventActor));
        this._leftCorner.connect('button-release-event', Lang.bind(this, this._onButtonReleaseEvent));
        this._leftCorner.connect('enter-event', Lang.bind(this, function() {
            this._onEnterEventCorner(true, cornerSize);
        }));
        this._leftCorner.connect('leave-event', Lang.bind(this, this._onLeaveEventCorner));
        this._rightCorner.connect('button-release-event', Lang.bind(this, this._onButtonReleaseEvent));
        this._rightCorner.connect('enter-event', Lang.bind(this, function() {
            this._onEnterEventCorner(false, cornerSize);
        }));
        this._rightCorner.connect('leave-event', Lang.bind(this, this._onLeaveEventCorner));
        this._ripple1l = new St.BoxLayout({ style_class: 'ripple-box', opacity: 0, visible: false });
        this._ripple2l = new St.BoxLayout({ style_class: 'ripple-box', opacity: 0, visible: false });
        this._ripple3l = new St.BoxLayout({ style_class: 'ripple-box', opacity: 0, visible: false });
        this._ripple1r = new St.BoxLayout({ style_class: 'ripple-box', opacity: 0, visible: false });
        this._ripple2r = new St.BoxLayout({ style_class: 'ripple-box', opacity: 0, visible: false });
        this._ripple3r = new St.BoxLayout({ style_class: 'ripple-box', opacity: 0, visible: false });
        this._ripple1r.add_style_pseudo_class('rtl');
        this._ripple2r.add_style_pseudo_class('rtl');
        this._ripple3r.add_style_pseudo_class('rtl');
        if (Main.layoutManager.uiGroup) {
            Main.layoutManager.uiGroup.add_actor(this._ripple1l);
            Main.layoutManager.uiGroup.add_actor(this._ripple2l);
            Main.layoutManager.uiGroup.add_actor(this._ripple3l);
            Main.layoutManager.uiGroup.add_actor(this._ripple1r);
            Main.layoutManager.uiGroup.add_actor(this._ripple2r);
            Main.layoutManager.uiGroup.add_actor(this._ripple3r);
        } else {
            Main.uiGroup.add_actor(this._ripple1l);
            Main.uiGroup.add_actor(this._ripple2l);
            Main.uiGroup.add_actor(this._ripple3l);
            Main.uiGroup.add_actor(this._ripple1r);
            Main.uiGroup.add_actor(this._ripple2r);
            Main.uiGroup.add_actor(this._ripple3r);
        }
    },

    _animRipple: function(cornerLeft, cornerSize, corner, ripple, delay, time, startScale, startOpacity, finalScale) {
        ripple._opacity = startOpacity;
        if (cornerLeft) {
            ripple.set_anchor_point_from_gravity(Clutter.Gravity.NORTH_WEST);
        } else {
            ripple.set_anchor_point_from_gravity(Clutter.Gravity.NORTH_EAST);
        }
        ripple.visible = true;
        ripple.opacity = 255 * Math.sqrt(startOpacity);
        ripple.scale_x = ripple.scale_y = startScale;
        let [x, y] = corner.get_transformed_position();
        ripple.x = x + cornerSize;
        ripple.y = y;
        Tweener.addTween(ripple, { _opacity: 0, scale_x: finalScale, scale_y: finalScale, delay: delay, time: time, transition: 'linear', onUpdate: function() {
            ripple.opacity = 255 * Math.sqrt(startOpacity); }, onComplete: function() {
                ripple.visible = false;
            }
        });
    },

    _onButtonReleaseEvent: function() {
        if (this._onClickOverviewToggle()) Main.overview.toggle();
        return true;
    },

    _onClickOverviewToggle: function() {
        if (Main.overview.animationInProgress) return false;
        if (this._activationTime === 0 || Date.now() / 1000 - this._activationTime > 0.5) return true;
        return false;
    },

    _onEnterEventCorner: function(cornerLeft, cornerSize) {
        if (!this._entered) {
            this._entered = true;
            if (!Main.overview.animationInProgress) {
                this._activationTime = Date.now() / 1000;
                if (cornerLeft) {
                    this._animRipple(true, 0, this._leftCorner, this._ripple1l, 0.0,  0.83, 0.25, 1.0, 1.5);
                    this._animRipple(true, 0, this._leftCorner, this._ripple2l, 0.05, 1.0,  0.0,  0.7, 1.25);
                    this._animRipple(true, 0, this._leftCorner, this._ripple3l, 0.35, 1.0,  0.0,  0.3, 1);
                } else {
                    this._animRipple(false, cornerSize, this._rightCorner, this._ripple1r, 0.0,  0.83, 0.25, 1.0, 1.5);
                    this._animRipple(false, cornerSize, this._rightCorner, this._ripple2r, 0.05, 1.0,  0.0,  0.7, 1.25);
                    this._animRipple(false, cornerSize, this._rightCorner, this._ripple3r, 0.35, 1.0,  0.0,  0.3, 1);
                }
                Main.overview.toggle();
            }
        }
        return false;
    },

    _onLeaveEventActor: function(actor, event) {
        if ((event.get_related() !== this._cornerLeft) && (event.get_related() !== this._cornerRight)) this._entered = false;
        return false;
    },

    _onLeaveEventCorner: function(actor, event) {
        if (event.get_related() !== this.actor) this._entered = false;
        return true;
    },

    destroy: function() {
        this.actor.destroy();
    }
});

const HotCorners = new Lang.Class({
    Name: 'HotCorners',

    _init: function() {
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = [
            this._settings.connect('changed::'+KEY_HOTCORNERS_SENSITIVITY, Lang.bind(this, this._updateHotCorners)),
            this._settings.connect('changed::'+KEY_SHOW_HOTCORNER_LEFT, Lang.bind(this, this._onShowHotCornerLeft)),
            this._settings.connect('changed::'+KEY_SHOW_HOTCORNER_RIGHT, Lang.bind(this, this._onShowHotCornerRight))
        ];
        this._connectHotCorners = global.screen.connect('monitors-changed', Lang.bind(this, this._updateHotCorners));
        this._updateHotCorners();
    },

    _clearHotCorners: function() {
        for (let i = 0; i < this._hotCorners.length; i++) {
            this._hotCorners[i].destroy();
        }
    },

    _createHotCorners: function() {
        this._hotCorners = [];
        let cornerSize = this._settings.get_int(KEY_HOTCORNERS_SENSITIVITY);
        for each(let monitor in Main.layoutManager.monitors) {
            let corners = new HotCornerSet(cornerSize, monitor.width);
            this._hotCorners.push(corners);
            Main.layoutManager.addChrome(corners.actor);
        }
    },

    _onShowHotCornerLeft: function() {
        let showHotCornerLeft = this._settings.get_boolean(KEY_SHOW_HOTCORNER_LEFT);
        for (let i = 0; i < this._hotCorners.length; i++) {
            this._hotCorners[i]._leftCorner.visible = showHotCornerLeft;
        }
    },

    _onShowHotCornerRight: function() {
        let showHotCornerRight = this._settings.get_boolean(KEY_SHOW_HOTCORNER_RIGHT);
        for (let i = 0; i < this._hotCorners.length; i++) {
            this._hotCorners[i]._rightCorner.visible = showHotCornerRight;
        }
    },

    _updateHotCorners: function() {
        if (this._hotCorners) this._clearHotCorners();
        this._createHotCorners();
        this._onShowHotCornerLeft();
        this._onShowHotCornerRight();
    },

    destroy: function() {
        global.screen.disconnect(this._connectHotCorners);
        for each(let connection in this._connectSettings) {
            this._settings.disconnect(connection);
        }
        this._clearHotCorners();
    }
});

function getExtension() {
    return new HotCorners();
}
